<?php

namespace Teknisa\ACL\Controllers;

use Teknisa\ACL\Services\PerfAcessParc as PerfAcessParcService;
use Teknisa\ACL\Util\AclUtilities;
use Zeedhi\Framework\Controller\Crud;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DataSource\DataSet;
use Teknisa\ACL\Util\InstanceProvider;
use Teknisa\Libs\Util\Environment as Environment;

class PerfAcessParc extends Crud{

    const IS_NEW = "__is_new";

    protected $dataSourceName = "perfAcessParcOperador";
    protected $dataSourceNameSave = "perfAcessParc";
    private   $environment;

    /** @var PerfAcessParcService */
    protected $perfAcessParcService;

    public function __construct(Manager $manager, PerfAcessParcService $perfAcessParcService, Environment $environment) {
        $this->perfAcessParcService = $perfAcessParcService;
        $this->environment = $environment;
        parent::__construct($manager);
    }

    public function find(DTO\Request\Filter $request, DTO\Response $response) {
        $request->getFilterCriteria()->addCondition(PerfAcessParcService::NRORG, $this->perfAcessParcService->getNrOrgFilter());
        parent::find($request, $response);
    }
    
    public function deleteOperator(DTO\Request $request, DTO\Response $response) {
            $rows       = $request->getDataSet()->getRows()[0];
            $labels = $rows['ACL_LABELS_TRADUCTION_LOG'];
            $nmPerfilAcess = InstanceProvider::getRepository()->getPerfAcessParc()->findNmPerfilAcess($rows['NRPERFILACESSO']);
        if($this->environment->getIdUtlPerfilFilial() != 'N' && $rows['SHOW_FULL_OPERATOR']){
            $data = InstanceProvider::getService()->getPerfAcessParc()->deleteOperatorPerfAccessFili($rows, $labels, $nmPerfilAcess[0]['NMPERFILACESSO']);
        } else {
            $arrayValores = [$labels['label'], $rows['CDOPERADOR'], $rows['NMOPERADOR'], $nmPerfilAcess[0]['NMPERFILACESSO'], $rows['NRPERFACESSPARC']];
            $arrayNomes = ['Tela', $labels['CDOPERADOR'], $labels['NMOPERADOR'], 'Perfil', 'NRPERFACESSPARC'];
            $descricao = AclUtilities::gerarDescricao($arrayValores, $arrayNomes);
            InstanceProvider::getLibLog()->logRegister($rows['NRPERFACESSPARC'], $descricao, '3', 'PERFACESSPARC', 'findPerfAcessParcOperador/remove');
            parent::delete($request, $response);
        }
    }

    public function save(DTO\Request\DataSet $request, DTO\Response $response) {
        $rows       = $request->getDataSet()->getRows()[0];
        if($this->environment->getIdUtlPerfilFilial() != 'N' && $rows['SHOW_FULL_OPERATOR']){
            self::saveOperadorPerfAccessFili($request, $response);    
        } else {
            self::saveOperador($request, $response);
        }
    }
    
    public function saveOperador($request, $response){
        $entityManager = InstanceProvider::getEntityManager();
        $connection = $entityManager->getConnection();
        $connection->beginTransaction();
        try {
            $rows = $request->getDataSet()->getRows();
            $labels = $rows[0]['ACL_LABELS_TRADUCTION_LOG'];
            foreach($rows as $row) {
                if(!$row['NRPARCNEGOCIO']) {
                    $businessPartnership = InstanceProvider::getService()->getOperator()->setBusinessPartnership(
                                    $row['NMOPERADOR'], $row['NRINSCRICAOPARC'], $row['IDATIVO'], $row['CDOPERINCLUSAO'],
                                    $row['CDTIPOINSCRICAO'], $row['CDOPERADOR']);
                    $row['NRPARCNEGOCIO'] = $businessPartnership['NRPARCNEGOCIO'];
                }
                if ($row[self::IS_NEW]) {
                    $row[PerfAcessParcService::NRPERFACESSPARC] = $this->perfAcessParcService->getMaxNrPerfilAcesso();
                }            
                $this->perfAcessParcService->setDefaultValues($row);
                InstanceProvider::getRepository()->getOperator()->saveAccessProfileByOperator($row);
                $nmPerfilAcess = InstanceProvider::getRepository()->getPerfAcessParc()->findNmPerfilAcess($row['NRPERFILACESSO']);
                $arrayValores = [$labels['label'], $row['CDOPERADOR'], $row['NMOPERADOR'], $nmPerfilAcess[0]['NMPERFILACESSO'], $row['NRPARCNEGOCIO'], $row['NRPERFILACESSO'], $row['DTVALIDADE']];
                $arrayNomes = ['Tela', 'Cod.Operador', 'Operador', 'Perfil', 'NRPARCNEGOCIO', 'NRPERFILACESSO', $labels['DTVALIDADE']];
                $descricao = AclUtilities::gerarDescricao($arrayValores, $arrayNomes);
                InstanceProvider::getLibLog()->logRegister($row['NRPERFACESSPARC'], $descricao, '1', 'PERFACESSPARC', '/perfAcessParc/save');
            }
            $entityManager->flush();
            $connection->commit();
        } catch (\Exception $e){
            $connection->rollBack();
            throw $e;
        }
    }
    
    public function saveOperadorPerfAccessFili(DTO\Request\DataSet $request, DTO\Response $response) {
        $row = $request->getDataSet()->getRows();
        $labels = $row[0]['ACL_LABELS_TRADUCTION_LOG'];
        $data = InstanceProvider::getService()->getPerfAcessParc()->saveOperadorPerfAccessFili($row, $labels);
    }

    public function findPerfAcessParcOperador(DTO\Request\Filter $request, DTO\Response $response)
    {
        $request->getFilterCriteria()->addCondition("NRORG", $this->environment->getCurrentOrganizationId());
        $conditions       = $request->getFilterCriteria()->getConditions();
        $showFullOperator = AclUtilities::getValueInFilterDataSourceByName($conditions, 'SHOW_FULL_OPERATOR', true);
        $orderBy          = $request->getFilterCriteria()->getOrderBy();
        $utlPerfilfilial  = $this->environment->getIdUtlPerfilFilial();

        if($utlPerfilfilial != 'N' && $showFullOperator) {
            $data = InstanceProvider::getService()->getPerfAcessParc()->findPerfAcessFilial($conditions, $orderBy);
        } else {
            $data = InstanceProvider::getService()->getPerfAcessParc()->findPerfAcessParcOperador($conditions, $orderBy);
        }
        $response->addDataSet(new DataSet('findPerfAcessParcOperador', $data));
    }

}