<?php

namespace Teknisa\ACL\Services;

use Teknisa\ACL\Repositories\PerfAcessParc as PerfAcessParcRepository;
use Teknisa\ACL\Util\InstanceProvider;
use Teknisa\Libs\Util\Environment;
use Teknisa\ACL\Util\AclUtilities;

class PerfAcessParc {

    const MAXNRPERFACESSPARC = "MAXNRPERFACESSPARC";
    const DTVALPERFILPARC = "DTVALPERFILPARC";
    const IDATIVO = "IDATIVO";
    const YES = "S";
    const NRPERFACESSPARC = "NRPERFACESSPARC";
    const NRORG = "NRORG";
    const NRORGINCLUSAO = "NRORGINCLUSAO";
    const NRORGULTATU = "NRORGULTATU";
    const DTINCLUSAO = "DTINCLUSAO";
    const DTULTATU = "DTULTATU";
    const CDOPERINCLUSAO = "CDOPERINCLUSAO";
    const CDOPERULTATU = "CDOPERULTATU";


    const ARRAY_FIELD_PAF = array("CDOPERADOR", "NMOPERADOR", "NRORG", "NRPERFILACESSO", "IDATIVO");

    const ARRAY_FIELD_PAPO = array( "CDOPERADOR", "NMOPERADOR", "NRPERFACESSPARC", "NRORG", "NRPARCNEGOCIO",
        "NRPERFILACESSO", "IDATIVO", "DTVALIDADE", "DTINCLUSAO", "DTULTATU", "NMOPERINCLUSAO", "NMOPERULTATU");

    /** @var PerfAcessParc */
    protected $perfAcessParcRepository;

    /** @var Environment */
    protected $environment;

    public function __construct(PerfAcessParcRepository $perfAcessParcRepository, Environment $environment) {
        $this->perfAcessParcRepository = $perfAcessParcRepository;
        $this->environment = $environment;
    }

    public function getMaxNrPerfilAcesso() {
        return InstanceProvider::getNewCode()->getCodeByTableName('PERFACESSPARC', $this->environment->getCurrentOrganizationId());
    }

    public function setDefaultValues(&$row) {
        $this->setNrOrgValues($row);
        $this->setDateValues($row);
        $this->setOperatorValues($row);
        $row[self::DTVALPERFILPARC] = null;
        $row[self::IDATIVO] = self::YES;
    }

    protected function setNrOrgValues(&$row) {
        $nrOrg = $this->getNrOrgFilter();
        $row[self::NRORG] = $this->setValueIfDontExist($row, self::NRORG, $nrOrg);
        $row[self::NRORGINCLUSAO] = $this->setValueIfDontExist($row, self::NRORGINCLUSAO, $nrOrg);
        $row[self::NRORGULTATU] = $nrOrg;
    }

    protected function setDateValues(&$row) {
        $time = new \DateTime();
        $timeFormatted = $time->format('d/m/Y');
        $row[self::DTINCLUSAO] = $this->setValueIfDontExist($row, self::DTINCLUSAO, $timeFormatted);
        $row[self::DTULTATU] = $timeFormatted;
    }

    protected function setOperatorValues(&$row) {
        $cdOperador = $this->environment->getCurrentUserId();
        $row[self::CDOPERINCLUSAO] = $this->setValueIfDontExist($row, self::CDOPERINCLUSAO, $cdOperador);
        $row[self::CDOPERULTATU] = $cdOperador;
    }

    protected function setValueIfDontExist($row, $property, $value) {
        return isset($row[$property]) ? $row[$property] : $value;
    }

    public function getNrOrgFilter() {
        return $this->environment->getCurrentOrganizationId();
    }
    
    public function deleteOperatorPerfAccessFili($conditions, $labels, $nmPerfilAcess){
        InstanceProvider::getRepository()->getPerfAcessParc()->deleteOperatorPerfAccessFili($conditions);
        $arrayValores = [$labels['label'], $conditions['CDOPERADOR'], $conditions['NMOPERADOR'], $nmPerfilAcess, $conditions['NRPERFILACESSO']];
        $arrayNomes = ['Tela', $labels['CDOPERADOR'], $labels['NMOPERADOR'], 'Perfil', $labels['NRPERFILACESSO']];
        $descricao = AclUtilities::gerarDescricao($arrayValores, $arrayNomes);
        InstanceProvider::getLibLog()->logRegister($conditions['CDOPERADOR'].'_'.$conditions['NRPERFILACESSO'], $descricao, '3', 'PERFACESSFILI', 'findPerfAcessParcOperador/remove');
        $operatorData = InstanceProvider::getRepository()->getOperator()->getOperatorData($conditions['CDOPERADOR'], $conditions['NRORG']);
        if($operatorData['CDPERFILSELECIONADO'] == $conditions['NRPERFILACESSO']){
            InstanceProvider::getRepository()->getOperator()->deleteRelatedBranchAndProfile($conditions['CDOPERADOR'], $conditions['NRORG']);
            $arrayValores = [$labels['label']];
            $arrayNomes = ['Tela'];
            if ($operatorData['CDPERFILSELECIONADO'] !== null) {
                array_push($arrayValores, null);
                array_push($arrayNomes, 'CDPERFILSELECIONADO');
            }
            if ($operatorData['CDFILIALSELECIONADA'] !== null) {
                array_push($arrayValores, null);
                array_push($arrayNomes, 'CDFILIALSELECIONADA');
            }
            $descricao = AclUtilities::gerarDescricao($arrayValores, $arrayNomes);
            InstanceProvider::getLibLog()->logRegister($conditions['CDOPERADOR'], $descricao, '2', 'OPERADOR', '/operator/remove');
        }
    }

    public function findPerfAcessParcOperador($conditions, $orderBy)
    {
        $orderByString = '';
        list($filters, $bindParams, $typeBindParams) = AclUtilities::createQueryFilteredAndBind($conditions, null,self::ARRAY_FIELD_PAPO);
        foreach($orderBy as $idx => $order) {
            if (AclUtilities::validadeOrder($idx, $order, self::ARRAY_FIELD_PAPO)) {
                if ($idx === 'DTVALIDADE') {
                    $orderByString .= ' ORDER BY TO_DATE(' . $idx . ') ' . $order . ' NULLS LAST';
                } else {
                    $orderByString .= ' ORDER BY ' . $idx . ' ' . $order;
                }
            }
        }
        $data = InstanceProvider::getRepository()->getPerfAcessParc()->findPerfAcessParcOperador($filters, $bindParams, $typeBindParams, $orderByString);
        foreach ($data as &$p){
            if(!$p['DTVALIDADE']){
                $p['DTVALIDADE'] = '';
            }
        }
        return $data;
    }
    
    public function findPerfAcessFilial($conditions, $orderBy)
    {
        $orderByString = '';
        list($filters, $bindParams, $typeBindParams) = AclUtilities::createQueryFilteredAndBind($conditions, null,self::ARRAY_FIELD_PAF);
        foreach($orderBy as $idx => $order) {
            if (AclUtilities::validadeOrder($idx, $order, self::ARRAY_FIELD_PAF)) {
                if ($idx === 'DTVALIDADE') {
                    $orderByString .= ' ORDER BY TO_DATE(' . $idx . ') ' . $order . ' NULLS LAST';
                } else {
                    $orderByString .= ' ORDER BY ' . $idx . ' ' . $order;
                }
            }
        }
        $data = InstanceProvider::getRepository()->getPerfAcessParc()->findPerfAcessFilial($filters, $bindParams, $typeBindParams, $orderByString);
      
        return $data;
    }
    
    public function saveOperadorPerfAccessFili($rows, $labels){
        foreach($rows as $row){
            foreach($row['CDFILIAL'] as $cdfilial){
                $data = InstanceProvider::getRepository()->getPerfAcessParc()->saveOperadorPerfAccessFili($row, $cdfilial, $labels);
                $pkPerfAcessFili = $cdfilial.'_'.$row['CDOPERADOR'].'_'.$row['NRPERFILACESSO'];
                $nmPerfilAcess = InstanceProvider::getRepository()->getPerfAcessParc()->findNmPerfilAcess($row['NRPERFILACESSO']);
                $arrayValores = [$labels['label'], $row['CDOPERADOR'], $row['NMOPERADOR'], $nmPerfilAcess[0]['NMPERFILACESSO'], $cdfilial, $row['NRPERFILACESSO']];
                $arrayNomes = ['Tela', 'Cod.Operador', 'Operador', 'Perfil', $labels['CDFILIAL'], $labels['NRPERFILACESSO']];
                $descricao = AclUtilities::gerarDescricao($arrayValores, $arrayNomes);
                InstanceProvider::getLibLog()->logRegister($pkPerfAcessFili, $descricao, '1', 'PERFACESSFILI', '/perfAcessParc/save');
            }
        }
    }

}